"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var redux_data_service_1 = require("redux-data-service");
var recompose_1 = require("recompose");
var lodash_1 = require("lodash");
var Query_1 = require("../Query");
var Helpers_1 = require("../Helpers");
var WithModelQuery_1 = require("../WithModelQuery");
var Average_1 = require("../Helpers/Average");
var DisplayPreviousPage = recompose_1.compose(recompose_1.setDisplayName("DisplayPreviousPage"), recompose_1.pure)((function (_a) {
    var queryManager = _a.queryManager, modelName = _a.modelName, ModelComponent = _a.modelComponent, modelComponentProps = _a.modelComponentProps;
    return queryManager.hasPreviousPage() && (React.createElement(Query_1.Query, { key: "page-" + queryManager.response.previousPage, modelName: modelName, query: queryManager.getPreviousPage() }, function (_a) {
        var query = _a.query;
        return (React.createElement(React.Fragment, null,
            React.createElement(DisplayPreviousPage, { queryManager: query, modelComponent: ModelComponent, modelComponentProps: modelComponentProps, modelName: modelName }),
            query.items.map(function (model) { return (React.createElement(ModelComponent, __assign({ key: model.id, model: model }, modelComponentProps))); })));
    }));
}));
var DefaultContentPlaceHolder = function (_a) {
    var height = _a.height;
    var style = {
        height: height,
    };
    return (React.createElement("div", { style: style }));
};
exports.InfiniteScroll = recompose_1.compose(recompose_1.setDisplayName("InfiniteScroll"), recompose_1.defaultProps({
    debounceTime: 200,
    disableVirtualScrolling: false,
    contentPlaceHolderComponent: DefaultContentPlaceHolder,
}), recompose_1.withProps({
    currentPageHeightRef: React.createRef(),
    previousPageHeightRef: React.createRef(),
    nextPageHeightRef: React.createRef(),
}), recompose_1.withStateHandlers({ pageHeightMap: {}, estimatedPageHeight: 0 }, {
    recordPageHeight: function () {
        return function (pageHeightMap) { return ({
            pageHeightMap: pageHeightMap,
            estimatedPageHeight: Average_1.average(Object.values(pageHeightMap)),
        }); };
    },
}), recompose_1.withState("query", "updateQuery", function (_a) {
    var query = _a.query, modelName = _a.modelName;
    return query instanceof redux_data_service_1.QueryBuilder
        ? query
        : new redux_data_service_1.QueryBuilder(modelName, query);
}), WithModelQuery_1.withModelQuery(), recompose_1.withPropsOnChange(["query", "estimatedPageHeight"], function (_a) {
    var query = _a.query, pageHeightMap = _a.pageHeightMap, estimatedPageHeight = _a.estimatedPageHeight;
    var previousPlaceHolderHeight = 0;
    var nextPlaceHolderHeight = 0;
    for (var x = query.response.currentPage - 2; x > 0; x--) {
        previousPlaceHolderHeight += x in pageHeightMap
            ? pageHeightMap[x]
            : estimatedPageHeight;
    }
    for (var x = query.response.currentPage + 2; x <= query.response.totalPages; x++) {
        nextPlaceHolderHeight += x in pageHeightMap
            ? pageHeightMap[x]
            : estimatedPageHeight;
    }
    return {
        previousPlaceHolderHeight: previousPlaceHolderHeight,
        nextPlaceHolderHeight: nextPlaceHolderHeight,
    };
}), recompose_1.withHandlers({
    updatePageHeightMap: function (_a) {
        var currentPageHeightRef = _a.currentPageHeightRef, nextPageHeightRef = _a.nextPageHeightRef, pageHeightMap = _a.pageHeightMap, previousPageHeightRef = _a.previousPageHeightRef, recordPageHeight = _a.recordPageHeight;
        return function (query) {
            var _a;
            var currentPageHeight = currentPageHeightRef.current.getBoundingClientRect().height;
            var previousPageHeight;
            var nextPageHeight;
            var updatedPageHeightMap = __assign({}, pageHeightMap, (_a = {}, _a[query.response.currentPage] = currentPageHeight, _a));
            if (query.hasPreviousPage()) {
                previousPageHeight = previousPageHeightRef.current.getBoundingClientRect().height;
                updatedPageHeightMap[query.response.previousPage] = previousPageHeight;
            }
            if (query.hasNextPage()) {
                nextPageHeight = nextPageHeightRef.current.getBoundingClientRect().height;
                updatedPageHeightMap[query.response.nextPage] = nextPageHeight;
            }
            recordPageHeight(updatedPageHeightMap);
            return updatedPageHeightMap;
        };
    },
}), recompose_1.withStateHandlers({ lastScrollTop: 0 }, {
    handleScroll: function (_a, _b) {
        var lastScrollTop = _a.lastScrollTop;
        var disableVirtualScrolling = _b.disableVirtualScrolling, estimatedPageHeight = _b.estimatedPageHeight, query = _b.query, updatePageHeightMap = _b.updatePageHeightMap, updateQuery = _b.updateQuery;
        return function (clientHeight, scrollHeight, currentScrollTop) {
            var scrollingDown = currentScrollTop > lastScrollTop;
            var currentScrollBottom = scrollHeight - currentScrollTop - clientHeight;
            if (!disableVirtualScrolling) {
                var updatedPageHeightMap = updatePageHeightMap(query);
                var nextPageToLoad = 1;
                var nextPageScrollTop = updatedPageHeightMap[1];
                while (nextPageScrollTop < (currentScrollTop + (clientHeight / 2)) && nextPageToLoad < query.response.totalPages) {
                    nextPageToLoad++;
                    nextPageScrollTop += nextPageToLoad in updatedPageHeightMap
                        ? updatedPageHeightMap[nextPageToLoad]
                        : estimatedPageHeight;
                }
                if (query.response.currentPage !== nextPageToLoad) {
                    updateQuery(query.query.page(nextPageToLoad));
                }
            }
            else if (scrollingDown && currentScrollBottom < (clientHeight / 2) && query.hasNextPage()) {
                updateQuery(query.getNextPage());
            }
            return { lastScrollTop: currentScrollTop };
        };
    },
}), recompose_1.withPropsOnChange(["debounceTime"], function (_a) {
    var debounceTime = _a.debounceTime, handleScroll = _a.handleScroll;
    return ({
        handleScrollDebounced: lodash_1.debounce(handleScroll, debounceTime),
        handleScrollThrottled: lodash_1.throttle(handleScroll, debounceTime),
    });
}), recompose_1.withHandlers({
    handleScrollPersistingEvent: function (_a) {
        var handleScrollDebounced = _a.handleScrollDebounced, handleScrollThrottled = _a.handleScrollThrottled;
        return function (event) {
            var clientHeight = event.target.clientHeight;
            var scrollHeight = event.target.scrollHeight;
            var scrollTop = event.target.scrollTop;
            handleScrollThrottled(clientHeight, scrollHeight, scrollTop);
            handleScrollDebounced(clientHeight, scrollHeight, scrollTop);
        };
    },
}), recompose_1.lifecycle({
    componentDidMount: function () {
        if (!this.props.disableVirtualScrolling) {
            this.props.updatePageHeightMap(this.props.query);
        }
    },
    componentWillUnmount: function () {
        this.props.handleScrollDebounced.cancel();
        this.props.handleScrollThrottled.cancel();
    },
}), Helpers_1.omitProps([
    "debounceTime",
    "handleScroll",
    "handleScrollDebounced",
    "handleScrollThrottled",
    "lastScrollTop",
    "recordPageHeight",
    "updateQuery",
    "updatePageHeightMap",
]), recompose_1.pure)(function (_a) {
    var ContainerComponent = _a.containerComponent, ContentPlaceHolder = _a.contentPlaceHolderComponent, currentPageHeightRef = _a.currentPageHeightRef, disableVirtualScrolling = _a.disableVirtualScrolling, estimatedPageHeight = _a.estimatedPageHeight, handleScrollPersistingEvent = _a.handleScrollPersistingEvent, ModelComponent = _a.modelComponent, modelComponentProps = _a.modelComponentProps, modelName = _a.modelName, nextPageHeightRef = _a.nextPageHeightRef, nextPlaceHolderHeight = _a.nextPlaceHolderHeight, pageHeightMap = _a.pageHeightMap, previousPageHeightRef = _a.previousPageHeightRef, previousPlaceHolderHeight = _a.previousPlaceHolderHeight, queryManager = _a.query, containerProps = __rest(_a, ["containerComponent", "contentPlaceHolderComponent", "currentPageHeightRef", "disableVirtualScrolling", "estimatedPageHeight", "handleScrollPersistingEvent", "modelComponent", "modelComponentProps", "modelName", "nextPageHeightRef", "nextPlaceHolderHeight", "pageHeightMap", "previousPageHeightRef", "previousPlaceHolderHeight", "query"]);
    return (React.createElement(ContainerComponent, __assign({}, containerProps, { onScroll: handleScrollPersistingEvent }),
        disableVirtualScrolling &&
            React.createElement(DisplayPreviousPage, { queryManager: queryManager, modelComponent: ModelComponent, modelComponentProps: modelComponentProps, modelName: modelName }),
        queryManager.hasPreviousPage() && !disableVirtualScrolling &&
            React.createElement(React.Fragment, null,
                React.createElement(ContentPlaceHolder, { height: previousPlaceHolderHeight }),
                React.createElement("span", { ref: previousPageHeightRef },
                    React.createElement(Query_1.Query, { modelName: modelName, query: queryManager.getPreviousPage(), loadingComponent: ContentPlaceHolder, loadingComponentProps: { height: pageHeightMap[queryManager.response.previousPage] || estimatedPageHeight } }, function (_a) {
                        var query = _a.query;
                        return (query.items.map(function (model) { return (React.createElement(ModelComponent, __assign({ key: model.id, model: model }, modelComponentProps))); }));
                    }))),
        React.createElement("span", { ref: currentPageHeightRef }, queryManager.items.map(function (model) { return (React.createElement(ModelComponent, __assign({ key: model.id, model: model }, modelComponentProps))); })),
        queryManager.hasNextPage() && (React.createElement(React.Fragment, null,
            React.createElement("span", { ref: nextPageHeightRef },
                React.createElement(Query_1.Query, { modelName: modelName, query: queryManager.getNextPage(), loadingComponent: ContentPlaceHolder, loadingComponentProps: { height: pageHeightMap[queryManager.response.nextPage] || estimatedPageHeight } }, function (_a) {
                    var query = _a.query;
                    return (query.items.map(function (model) { return (React.createElement(ModelComponent, __assign({ key: model.id, model: model }, modelComponentProps))); }));
                })),
            !disableVirtualScrolling &&
                React.createElement(ContentPlaceHolder, { height: nextPlaceHolderHeight })))));
});
